define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-class",
    "dojo/keys",
    "dgrid/Keyboard",
    // Resources
    "dojo/text!./templates/ProjectPreviewSelector.html",
    "epi/i18n!epi/nls/episerver.cms.components.project",
    // Parent class
    "epi-cms/project/ProjectSelectorGrid",
    // Widgets in template
    "dijit/form/ToggleButton"
], function(
    declare,
    lang,
    domClass,
    keys,
    Keyboard,
    // Resources
    template,
    localization,
    // Parent class and mixins
    ProjectSelectorGrid
) {

    return declare([ProjectSelectorGrid], {
        // summary:
        //      A project selector for the project preview selector that extends the template to include
        //      a header and the default none option.
        // tags:
        //      internal

        // templateString: [protected] String
        //      A string that represents the widget template.
        templateString: template,

        // header: [protected] String
        //      A string that will be displayed in the header of the selector.
        header: localization.preview.header,

        _setHeaderAttr: { node: "headerNode", type: "innerText" },

        buildRendering: function() {
            // summary:
            //      Construct the UI for this widget.
            // tags:
            //      protected
            this.inherited(arguments);

            this.defaultOption.set({
                iconClass: "epi-radio-icon--secondary",
                label: localization.preview.defaultoption
            });

            domClass.add(this.domNode, "epi-project-preview");
            domClass.add(this.gridNode, "epi-plain-grid--no-border");
        },

        postCreate: function() {
            // summary:
            //      Processing after the DOM fragment is created. Sets up event handlers.
            // tags:
            //      protected
            this.inherited(arguments);

            this.grid.set("noDataMessage", localization.placeholder.noprojectscreated);

            // Setup query options for the query engine to ignore the status parameter.
            this.grid.set("queryOptions", { ignore: ["status"] });

            // Setup event handlers required for the selector.
            this.own(
                // Handle selection of the default option.
                this.defaultOption.on("click", lang.hitch(this, "_defaultOptionSelected")),
                // Handle key events on the default option in order to manage moving focus.
                this.defaultOption.on("keypress", lang.hitch(this, "_defaultOptionKeypress")),
                // Handle key events on the grid in order to manage moving focus.
                this.grid.addKeyHandler(keys.UP_ARROW, lang.hitch(this, "_gridNavigateUp")),
                this.grid.addKeyHandler(keys.PAGE_UP, lang.hitch(this, "_gridNavigateUp"))
            );

            // Override the default handler for HOME key event since we don't want to execute the default handler.
            this.grid.keyMap[keys.HOME] = lang.hitch(this, "_gridNavigateHome");
        },

        _defaultOptionSelected: function() {
            // summary:
            //      The callback method for when the default option is selected. Set the
            //      selected project to null.
            // tags:
            //      private
            this.set("selectedProject", null);
        },

        _defaultOptionKeypress: function(e) {
            // summary;
            //      Event handler for key presses on the default option to take care of moving focus to the grid.
            // tags:
            //      private
            var method;

            switch (e.keyCode) {
                case keys.DOWN_ARROW:
                    method = Keyboard.moveFocusHome;
                    break;
                case keys.PAGE_DOWN:
                    method = Keyboard.moveFocusPageDown;
                    break;
                case keys.END:
                    method = Keyboard.moveFocusEnd;
                    break;
                default:
                    return;
            }

            // Apply the correct method to the grid giving the event as argument.
            method.apply(this.grid, [e]);
        },

        _gridNavigateUp: function(e) {
            // summary;
            //      Event handler for page up and arrow up key presses on the grid to take care of
            //      moving focus to the default option when the first row is selected in the grid.
            // tags:
            //      private
            var grid = this.grid,
                row = grid.row(e),
                previousRow = grid.up(row, 1, true);

            // If row and previous row are the same then the first row is selected.
            if (row.id === previousRow.id) {
                this.defaultOption.focus();
            }
        },

        _gridNavigateHome: function() {
            // summary:
            //      Event handler for a HOME key press on the grid to take care of moving
            //      focus to the default option.
            // tags:
            //      private
            this.defaultOption.focus();
        },

        _setSelectedProjectAttr: function() {
            // summary:
            //      Sets the selected project and ensures the UI is in the correct state.
            // tags:
            //      protected
            this.inherited(arguments);

            this._updateView();
        },

        _projectSelected: function() {
            // summary:
            //      The callback method for when a project is selected. Ensures the UI is
            //      in the correct state.
            // tags:
            //      protected
            this.inherited(arguments);

            this._updateView();
        },

        _updateView: function() {
            // summary:
            //      Updates the view to ensure it matches the current model state.
            // tags:
            //      private

            // The default option should be checked if there is no selected project.
            this.defaultOption.set("checked", !this.selectedProject);
        }
    });
});
