﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/Deferred",
    "dojo/promise/all",
    "dojo/Evented",

// epi
    "epi/dependency",
    "epi/shell/xhr/ErrorHandler",
    "epi-cms/core/ContentReference"
],

function (
// dojo
    declare,
    lang,
    when,
    Deferred,
    all,
    Evented,

// epi
    dependency,
    ErrorHandler,
    ContentReference
) {

    return declare([Evented], {
        // summary:
        //      A service for interacting with projects
        // tags:
        //      internal

        // projectItemStore: [readonly] Store
        //      A REST store for interacting with project items.
        projectItemStore: null,

        // projectStore: [readonly] Store
        //      A REST store for interacting with projects.
        projectStore: null,

        constructor: function(params) {
            declare.safeMixin(this, params);

            this.projectItemStore = this.projectItemStore || dependency.resolve("epi.storeregistry").get("epi.cms.project.item");
            this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
        },

        getProjectItemsForContent: function(contentLinks) {
            // summary:
            //      Get the associated projects items for the given content links
            //
            // contentLinks: String|Array
            //      The content links to get the project items for for
            // tags:
            //      internal

            if(!(contentLinks instanceof Array)){
                contentLinks = [contentLinks];
            }

            return when(this.projectItemStore.query({contentLinks: contentLinks}));
        },

        getProjectsForContent: function(contentLinks) {
            // summary:
            //      Get the associated projects for the given content links
            //
            // contentLinks: String|Array
            //      The content links to get the projects for
            // tags:
            //      internal

            var self = this;

            if(!(contentLinks instanceof Array)){
                contentLinks = [contentLinks];
            }

            return this.getProjectItemsForContent(contentLinks).then(function(projectItems) {
                // If no project items where found return an empty array
                if (projectItems.length === 0) {
                    return new Deferred().resolve([]);
                }

                return self.getProjects(projectItems);
            });
        },

        canAddContent: function(projectId, contentLinks) {
            // summary:
            //      Verifies that the given content links can be added to the project
            //
            // projectId: Number
            //      The project id to check
            // contentLinks: Array
            //      The content links to add
            // returns: Promise
            //      A promise with the result from the server
            // tags:
            //      internal

            return this.projectStore.executeMethod("CanAddContent", projectId, {contentReferences: contentLinks});
        },

        getProjects: function(projectItems) {
             // summary:
            //      Get the projects for the project items
            //
            // projectItems: int[]
            //      The project items to get the projects for
            // tags:
            //      internal

            var projectStore = this.projectStore;
            var projectPromises = projectItems.map(function(projectItem) {
                return projectStore.get(projectItem.projectId);
            });

            return all(projectPromises);
        },

        addProjectItems: function (projectId, contentReferences) {
            // summary:
            //      Adds the given content references as items of the project.
            //
            // projectId: Number
            //      The target project for adding items to.
            // contentReferences: ContentReference[]
            //      An array of references to content items to add to the project.
            // tags:
            //      public

            // Ensure the content reference we received are valid.
            contentReferences = contentReferences.filter(function (reference) {
                if (ContentReference.isContentReference(reference)) {
                    return true;
                }
                console.warn("epi-cms/project/ProjectService: the given value '" + reference + "' is not a valid content reference.");
            });

            var emit = lang.hitch(this, "emit"),
                store = this.projectItemStore,
                promises = contentReferences.map(function (reference) {
                    return store.add({ contentLink: reference, projectId: projectId });
                }),
                result = all(promises);

            result.then(function(items) {
                    emit("items-added", {
                        items: items
                    });
                })
                .otherwise(ErrorHandler.forXhr);

            return result;
        },

        removeProjectItems: function (projectId, projectItems) {
            // summary:
            //      Removes a project item from a project.
            //
            // projectId: Number
            //      The target project for adding items to.
            // projectItems: ProjectItem[]
            //      An array of project item entities to remove from the
            // tags:
            //      public

            var emit = lang.hitch(this, "emit"),
                remove = lang.hitch(this.projectItemStore, "remove"),
                promises = projectItems.map(function(item) { return remove(item.id); }),
                result = all(promises);

            result.then(function(items) {
                emit("items-removed", {
                    items: projectItems
                });
            }).otherwise(ErrorHandler.forXhr);

            return result;
        },

        markAsReadyToPublish: function (id) {
            // summary:
            //      Mark the project item as ready to publish
            // returns: Promise
            //      A promise that resolves when the refresh project has completed.
            // tags:
            //      public

            var promise = when(this.projectItemStore.executeMethod("ReadyToPublish", id));
            promise.otherwise(ErrorHandler.forXhr);
            return promise;
        },

        isPartOfDelayedPublishedProject: function(contentLink) {
            // summary:
            //      Determines if the content is part of any project that is set to delayed publish
            // contentLink: String
            //      The content link to check
            // returns:
            //      Promise
            // tags:
            //      public

            return this.getProjectsForContent([contentLink])
                .then(function(projects){
                    return projects.some(function(project) {
                        return project.status === "delayedpublished";
                    });
                });
        },

        publishProject: function (projectId, delayPublishUntil) {
            // summary:
            //      Publishes a project with given id right away or delay publishing until given delayPublish date. 
            // projectId: Number
            //      Id of the project to publish
            // delayPublishUntil: Date?
            //      If supplied, then the project will be delay published.
            // returns: Promise
            // tags:
            //      public

            var store = this.projectStore;

            if (isNaN(projectId)) {
                throw new Error("projectId is not a number");
            }

            var result = when(store.executeMethod("Publish", projectId, { delayPublishUntil: delayPublishUntil }));
            result.otherwise(ErrorHandler.forXhr);

            // we need to patch the store to update cache and notify the observers.
            return result.then(store.patchCache.bind(store));
        },

        reactivateProject: function(projectId) {
            // summary:
            //      Reactivates a project that has been scheduled for publishing.
            // projectId: Number
            //      The ID of the project to reactivate
            // returns: Promise
            // tags:
            //      public

            var store = this.projectStore;

            if (isNaN(projectId)) {
                throw new Error("The given project ID is not a number.");
            }

            var result = when(store.executeMethod("Reactivate", projectId));
            result.otherwise(ErrorHandler.forXhr);

            // we need to patch the store to update cache and notify the observers.
            return result.then(store.patchCache.bind(store));
        }
    });
});
