define([
    "dojo/_base/declare",
    "dojo/on",
    "dojo/query",
    "dojo/string",


    "dijit/DropDownMenu",
    "dijit/registry",

    "epi/shell/command/builder/MenuAssembler",
    "epi/shell/command/builder/MenuBuilder",

    "./command/PublishMenuButtonBuilder",

    "dojo/text!./templates/PublishMenu.html"

], function (
    declare,
    on,
    query,
    string,

    DropDownMenu,
    registry,

    MenuAssembler,
    MenuBuilder,
    PublishMenuButtonBuilder,

    template
    ) {

    return declare([DropDownMenu], {
        // summary:
        //      Composite menu for displaying a publish button and general commands for a content item.
        //      It also sports two custom regions for information text, links or widgets.
        //
        // description:
        //      Composite popup menu containing two regions for exposing commands and two custom regions
        //      for showing general markup, text or widgets.
        //
        //      A command can be placed in the top, publish region, by setting the command category to
        //      "publishmenu-primary". The primary commands will be represented by buttons and won't close
        //      the menu when executed.
        //      By setting the command category to "publishmenu", the command will be added to the lower
        //      region as regular menu options.
        //
        //      The two custom regions, "primarySection" and "statusSection", is populated from the named
        //      sections in source content.
        //
        //      Markup example:
        //
        // |    <div id="publishMenu" data-dojo-type="epi-cms/project/PublishMenu">
        // |        <span data-epi-section="primarySection">Hello World!</span>
        // |        <span data-epi-section="statusSection">
        // |            <ul>
        // |                <li>Good evening!</li>
        // |            </ul>
        // |        </span>
        // |    </div>
        //
        // tags:
        //      internal

        templateString: template,

        buildRendering: function (){
            this.inherited(arguments);

            var configuration = [{
                builder: new PublishMenuButtonBuilder({
                    settings: { "class": "epi-fullWidthButton Sweet", isExecutingClass: "epi-loading" }
                }),
                category: "publishmenu-primary",
                target: this.publishMenuNode
            }, {
                builder: new MenuBuilder(),
                category: "publishmenu",
                target: this.projectActionMenu
            }];

            this.own(this._menuAssembler = new MenuAssembler({
                configuration: configuration,
                commandSource: this.commandSource
            }));

            // The PublishMenuButtonBuilder makes sure that any button being "undisplayed" while focused
            // emits an event before focus is lost.
            this.own(on(this.domNode, "beforeFocusedButtonRemoved", this.domNode.focus));
        },

        childSelector: function(/*DOMNode*/ node){
            // summary:
            //		Selector (passed to on.selector()) used to identify MenuItem child widgets, but exclude inert children
            //		like MenuSeparator.
            // tags:
            //		protected

            // Called by _MenuBase. to determine which items are menu options. Exclude the publish button,
            // since buttons handle their own click execution
            var widget = registry.byNode(node);
            return widget && widget.focus && (node.parentNode === this.projectActionMenu);
        },

        getChildren: function() {
            // summary:
            //      Returns publish button and menu options. Used by _KeyNavContainer to navigate between nodes.

            return [].concat(
                registry.findWidgets(this.publishMenuNode),
                registry.findWidgets(this.projectActionMenu));
        },

        _setCommandSourceAttr: function(commandSource) {
            // summary:
            //      Set the command source which provides commands to the menu

            this._menuAssembler.set("commandSource", commandSource);
        },

        _getCommandSourceAttr: function() {
            // summary:
            //      Gets the command source from the internal menu assembler

            return this._menuAssembler.get("commandSource");
        },

        _fillContent: function(/*DomNode*/ source) {
            // summary:
            //      Overridden from _TemplatedMixin to populate the templated sections topText and middleText
            // tags:
            //		protected

            var sections = ["primarySection", "statusSection"];
            sections.forEach(function(section) {
                var targetNode = this[section],
                    sourceNodes = query(string.substitute('[data-epi-section=${0}]', [section]), source);

                sourceNodes.forEach(targetNode.appendChild, targetNode);
            }, this);
        }
    });
});



